@@ -239,6 +239,7 @@ def bof(code, G, eps):
 
 class LGraph(gsp.graphs.Graph):
     def __init__(self, L, gtype='unknown', lap_type='combinatorial'):
+        self.logger = gsp.utils.build_logger(__name__)
         self.L = L
         self.lap_type = lap_type
         self.N = L.shape[0]
@@ -246,6 +247,10 @@ class LGraph(gsp.graphs.Graph):
 
 class Hypergraph(LGraph):
     def __init__(self, I, gtype='unknown', lap_type='combinatorial'):
+        self.logger = gsp.utils.build_logger(__name__)
+        self._W = None
+        self._d = None
+        self._dw = None
         self.I = sparse.csr_matrix(I)
         self.lap_type = lap_type
         self.N = I.shape[0]
@@ -264,24 +269,30 @@ class Hypergraph(LGraph):
     def compute_laplacian(self, lap_type):
         self.L = self.I @ self.I.transpose()
         if lap_type == 'normalized':
-            D_inv_sqrt = np.diagflat(1.0/np.sqrt(self.d))
+            D_inv_sqrt = np.diagflat(1.0/np.sqrt(self.dw))
             self.L = D_inv_sqrt @ self.L @ D_inv_sqrt
         self.L = sparse.csc_matrix(self.L)
 
     @property
     def W(self):
-        if not hasattr(self, '_W'):
-            w = (self.I @ self.I.transpose()).todense()
+        if self._W is None:
+            w = (self.I @ self.I.transpose()).toarray()
             np.fill_diagonal(w, 0)
             self._W = sparse.lil_matrix(w)
         return self._W
 
     @property
     def d(self):
-        if not hasattr(self, '_d'):
-            self._d = np.asarray(np.sum(self.I != 0, axis=1)).squeeze()
+        if self._d is None:
+            self._d = np.asarray(np.sum((self.I != 0).toarray(), axis=1)).squeeze()
         return self._d
 
+    @property
+    def dw(self):
+        if self._dw is None:
+            self._dw = np.asarray(np.sum(np.power(self.I.toarray(), 2), axis=1)).squeeze()
+        return self._dw
+
 class GWHeat(gsp.filters.Filter):
     """
     Heat kernel used by GraphWave.
